describe("ValidationEngine", function(){
	var validationEngine;
	var validationNameToValidationFunction;
	var applicationContext;
	
	beforeEach(function () {
		module("ZeedhiApp");
		module("ZeedhiServices");
		
		
		inject(function (validateEngine, ApplicationContext) {
			validationEngine = validateEngine;
			applicationContext = ApplicationContext;
			
			applicationContext.registerInstance('testObject', {
				testFunction: function () {
					return "uabafet";
				}
			});
			
			validationNameToValidationFunction = {
				"min": validationEngine.min, "max": validationEngine.max, "range": validationEngine.range, "maxLength": validationEngine.maxLength,
				"minLength": validationEngine.minLength, "lengthRange": validationEngine.lengthRange, "equalsTo": validationEngine.equalsTo,
				"required": validationEngine.required, "mail": validationEngine.mail, "url": validationEngine.url, "dateIsValid": validationEngine.dateIsValid,
				"dateTimeIsValid": validationEngine.dateTimeIsValid, "dateGreaterThan": validationEngine.dateGreaterThan,
				"dateLessThan": validationEngine.dateLessThan, "hourGreaterThan": validationEngine.hourGreaterThan,
				"hourLessThan": validationEngine.hourLessThan, "extensions": validationEngine.extensions, "maxSize": validationEngine.maxSize,
				"minSize": validationEngine.minSize
			};
		});
	});

	it('formatArrayMessage: must create an error message text', function(){
		var message = "The file is too large:";
		var name = "my-picture.jpg";
		expect(validationEngine.formatArrayMessage(message, name, 1, 1)).toBe('The file is too large: "my-picture.jpg"');
		expect(validationEngine.formatArrayMessage(message, name, 1, 2)).toBe('The file is too large: "my-picture.jpg"\n');
		expect(validationEngine.formatArrayMessage(message, name, 2, 1)).toBe('The file is too large: "my-picture.jpg"');
	});

	it('getMappedValidations: must return the validationNameToValidationFunction array', function(){
		expect(validationEngine.getMappedValidations()).toEqual(validationNameToValidationFunction);
	});

	it('hasValidation: checks if the ValidationEngine has the specified validation', function(){
		expect(validationEngine.hasValidation('minSize')).toBe(true);
		expect(validationEngine.hasValidation('inexistent')).toBe(false);
	});

	it('registerValidation: checks if the validation was successful registered', function(){
		validationEngine.registerValidation('newValidation', function(){});
		expect(validationEngine.hasValidation('newValidation')).toBe(true);	
		expect(function(){ validationEngine.registerValidation('newValidation', function(){}); }).toThrow();
		expect(function(){ validationEngine.registerValidation(''); }).toThrow();
	});
	
	it("minLength: Should be valid if the value length is greater or equal than the limit", function () {
		expect(validationEngine.minLength("sss", {length: 3}).valid).toBeTruthy();
		expect(validationEngine.minLength("sss", {length: 2}).valid).toBeTruthy();
	});
	
	it("minLength: Should not be valid if the value length is lower than the limit", function () {
		expect(validationEngine.minLength("sss", {length: 4}).valid).toBeFalsy();
		expect(validationEngine.minLength(null, {length: 1}).valid).toBeFalsy();
		expect(validationEngine.minLength(undefined, {length: 1}).valid).toBeFalsy();
	});
	
	it("maxLength: Should be valid if the value length is lower or equal than the limit", function () {
		expect(validationEngine.maxLength("sss", {length: 3}).valid).toBeTruthy();
		expect(validationEngine.maxLength("sss", {length: 4}).valid).toBeTruthy();
		expect(validationEngine.maxLength(null, {length: 1}).valid).toBeTruthy();
		expect(validationEngine.maxLength(undefined, {length: 1}).valid).toBeTruthy();
	});
	
	it("maxLength: Should not be valid if the value length is greater than the limit", function () {
		expect(validationEngine.maxLength("sss", {length: 2}).valid).toBeFalsy();
	});
	
	it("required: Should not be valid if is an empty string", function () {
		var validationResult = validationEngine.required("", {});
		expect(validationResult.valid).toBeFalsy();
	});
	
	it("required: Should not be valid if is an empty array", function () {
		var validationResult = validationEngine.required([], {});
		expect(validationResult.valid).toBeFalsy();
	});
	
	it("required: Should be valid if the string have a content, even if the value of the string is 'false'", function () {
		var validationResult = validationEngine.required("uabafet", {});
		expect(validationResult.valid).toBeTruthy();
		
		validationResult = validationEngine.required("false", {});
		expect(validationResult.valid).toBeTruthy();
	});
	
	it("equalsTo: Should be valid if the value is equal to the text", function () {
		var validationResult = validationEngine.equalsTo("uabafet", {text: "uabafet"});
		expect(validationResult.valid).toBeTruthy();
	});
	
	it("equalsTo: Should not be valid if the values are not equals", function () {
		var validationResult = validationEngine.equalsTo("uabafet", {text: "ss"});
		expect(validationResult.valid).toBeFalsy();
	});
	
	it("equalsTo: Should be valid if the value is equal to the text expression result", function () {
		var validationResult = validationEngine.equalsTo("uabafet", {text: "@testObject.testFunction();@"});
		console.log(validationResult);
		expect(validationResult.valid).toBeTruthy();
	});
	
	it("equalsTo: Should not be valid if the value is not equal to the text expression result", function () {
		applicationContext.testObject.testFunction = function () {
			return 'different';
		};
		var validationResult = validationEngine.equalsTo("uabafet", {text: "@testObject.testFunction();@"});
		expect(validationResult.valid).toBeFalsy();
	});
	
	it("mail: Should be valid if there is no value", function () {
		expect(validationEngine.mail("", {}).valid).toBeTruthy();
	});
	
	it("mail: Should not be valid if the value is not an email", function () {
		expect(validationEngine.mail("daadsadsds", {}).valid).toBeFalsy();
		expect(validationEngine.mail("daadsadsds@aaa", {}).valid).toBeFalsy();
		expect(validationEngine.mail("daadsadsd/s@aaa.com", {}).valid).toBeFalsy();
		expect(validationEngine.mail("daadsadsd?s@aaa.com", {}).valid).toBeFalsy();
	});
	
	it("mail: Should be valid if it is a valid email", function () {
		expect(validationEngine.mail("adassdasdas@dasdsadsa.com", {}).valid).toBeTruthy();
	});
	
	it("url: Should be valid if it is an http, https, ftp or ssh url", function () {
		expect(validationEngine.url("http://dasdsasa.adads.com", {}).valid).toBeTruthy();
		expect(validationEngine.url("https://dasdsasa.adads.com", {}).valid).toBeTruthy();
		expect(validationEngine.url("ftp://dasdsasa.adads.com", {}).valid).toBeTruthy();
		expect(validationEngine.url("ssh://dasdsasa.adads.com", {}).valid).toBeTruthy();
	});
	
});
